"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createApiModels = exports.getFakeObject = exports.getColumOptions = exports.jsonSchemaResponse = exports.jsonSchemaBody = exports.getOptionsFiled = exports.getRelationsOptions = void 0;
const swagger_1 = require("@nestjs/swagger");
const class_transformer_1 = require("class-transformer");
const typeorm_1 = require("typeorm");
const class_validator_1 = require("class-validator");
const utils_1 = require("../utils");
function getRelationsOptions(entity) {
    return (0, typeorm_1.getMetadataArgsStorage)()
        .relations.filter((i) => i.target === entity)
        .reduce((acum, item) => {
        acum[item.propertyName] = item.type;
        return acum;
    }, {});
}
exports.getRelationsOptions = getRelationsOptions;
function getOptionsFiled(entity) {
    return (0, class_validator_1.getMetadataStorage)()
        .getTargetValidationMetadatas(entity, '', false, true)
        .reduce((acum, i) => {
        if (i.constraintCls) {
            const option = (0, class_validator_1.getMetadataStorage)()
                .getTargetValidatorConstraints(i.constraintCls)
                .filter((i) => i.name === class_validator_1.IS_NOT_EMPTY || i.name === class_validator_1.IS_EMPTY)
                .map((i) => i.name);
            if (option.length > 0) {
                if (option.length === 1) {
                    acum[i.propertyName] = option[0];
                }
                else {
                    acum[i.propertyName] = option.includes(class_validator_1.IS_NOT_EMPTY)
                        ? class_validator_1.IS_NOT_EMPTY
                        : class_validator_1.IS_EMPTY;
                }
            }
        }
        return acum;
    }, {});
}
exports.getOptionsFiled = getOptionsFiled;
function jsonSchemaBody(entity, currentField) {
    const optionsFiled = getOptionsFiled(entity);
    const relationsOptions = getRelationsOptions(entity);
    return {
        data: {
            type: 'object',
            properties: {
                type: {
                    type: 'string',
                    enum: [(0, utils_1.camelToKebab)((0, utils_1.getEntityName)(entity))],
                },
                attributes: {
                    type: 'object',
                    properties: currentField
                        .filter((item) => optionsFiled[item] !== class_validator_1.IS_EMPTY)
                        .reduce((acum, item) => {
                        const typeMetadata = Reflect.getMetadata('design:type', entity['prototype'], item);
                        let value = {};
                        switch (typeMetadata) {
                            case Array:
                                value = {
                                    type: 'array',
                                    items: {
                                        type: 'string',
                                    },
                                };
                                break;
                            case Date:
                                value = {
                                    format: 'date-time',
                                    type: 'string',
                                };
                                break;
                            case Number:
                                value = {
                                    type: 'integer',
                                };
                                break;
                            case Boolean:
                                value = {
                                    type: 'boolean',
                                };
                                break;
                            default:
                                value = {
                                    type: 'string',
                                };
                        }
                        acum[item] = value;
                        return acum;
                    }, {}),
                    required: currentField.filter((i) => optionsFiled[i] === class_validator_1.IS_NOT_EMPTY),
                },
                relationships: {
                    type: 'object',
                    properties: Object.keys(relationsOptions).reduce((acum, item) => {
                        const dataItem = {
                            type: 'object',
                            properties: {
                                type: {
                                    type: 'string',
                                    enum: [
                                        // @ts-ignore
                                        (0, utils_1.camelToKebab)((0, utils_1.getEntityName)(relationsOptions[item]())),
                                    ],
                                },
                                id: {
                                    type: 'string',
                                },
                            },
                            required: ['type', 'id'],
                        };
                        const dataArray = {
                            type: 'array',
                            items: dataItem,
                        };
                        acum[item] = {
                            type: 'object',
                            properties: {
                                data: Reflect.getMetadata('design:type', entity['prototype'], item) === Array
                                    ? dataArray
                                    : dataItem,
                            },
                            required: ['data'],
                        };
                        return acum;
                    }, {}),
                    required: Object.keys(relationsOptions).filter((i) => optionsFiled[i] === class_validator_1.IS_NOT_EMPTY),
                },
            },
            required: ['type', 'attributes'],
        },
    };
}
exports.jsonSchemaBody = jsonSchemaBody;
function jsonSchemaResponse(entity, currentField) {
    const relationsOptions = getRelationsOptions(entity);
    return {
        type: 'object',
        properties: {
            meta: {
                type: 'object',
                properties: {
                    debug: {
                        type: 'object',
                        properties: {
                            callQuery: {
                                type: 'integer',
                            },
                            prepareParams: {
                                type: 'integer',
                            },
                            transform: {
                                type: 'integer',
                            },
                        },
                    },
                },
            },
            data: {
                type: 'object',
                properties: {
                    type: {
                        type: 'string',
                        enum: [(0, utils_1.camelToKebab)((0, utils_1.getEntityName)(entity))],
                    },
                    id: {
                        type: 'string',
                    },
                    attributes: {
                        type: 'object',
                        properties: currentField.reduce((acum, item) => {
                            const typeMetadata = Reflect.getMetadata('design:type', entity['prototype'], item);
                            let value = {};
                            switch (typeMetadata) {
                                case Array:
                                    value = {
                                        type: 'array',
                                        items: {
                                            type: 'string',
                                        },
                                    };
                                    break;
                                case Date:
                                    value = {
                                        format: 'date-time',
                                        type: 'string',
                                    };
                                    break;
                                case Number:
                                    value = {
                                        type: 'integer',
                                    };
                                    break;
                                case Boolean:
                                    value = {
                                        type: 'boolean',
                                    };
                                    break;
                                default:
                                    value = {
                                        type: 'string',
                                    };
                            }
                            acum[item] = value;
                            return acum;
                        }, {}),
                    },
                    relationships: {
                        type: 'object',
                        properties: Object.keys(relationsOptions).reduce((acum, item) => {
                            const dataItem = {
                                type: 'object',
                                properties: {
                                    type: {
                                        type: 'string',
                                        enum: [
                                            // @ts-ignore
                                            (0, utils_1.camelToKebab)((0, utils_1.getEntityName)(relationsOptions[item]())),
                                        ],
                                    },
                                    id: {
                                        type: 'string',
                                    },
                                },
                                required: ['type', 'id'],
                            };
                            const dataArray = {
                                type: 'array',
                                items: dataItem,
                            };
                            acum[item] = {
                                type: 'object',
                                properties: {
                                    data: Reflect.getMetadata('design:type', entity['prototype'], item) === Array
                                        ? dataArray
                                        : dataItem,
                                    links: {
                                        type: 'object',
                                        properties: {
                                            self: {
                                                type: 'string',
                                            },
                                            related: {
                                                type: 'string',
                                            },
                                        },
                                        required: ['self', 'related'],
                                    },
                                },
                                required: ['links'],
                            };
                            return acum;
                        }, {}),
                        required: Object.keys(relationsOptions),
                    },
                    links: {
                        type: 'object',
                        properties: {
                            self: {
                                type: 'string',
                            },
                        },
                        required: ['self'],
                    },
                },
                required: ['type', 'id', 'attributes', 'links', 'relationships'],
            },
        },
        required: ['meta', 'data'],
    };
}
exports.jsonSchemaResponse = jsonSchemaResponse;
function getColumOptions(entity) {
    return (0, typeorm_1.getMetadataArgsStorage)()
        .columns.filter((i) => i.target === entity && !i.options.primary)
        .reduce((acum, item) => {
        if (item.options.type) {
            acum[item.propertyName] = item.options.type;
        }
        return acum;
    }, {});
}
exports.getColumOptions = getColumOptions;
function getFakeObject() { }
exports.getFakeObject = getFakeObject;
function createApiModels(entity) {
    const columOptions = getColumOptions(entity);
    const relationsOptions = getRelationsOptions(entity);
    const fakeObject = Object.keys(columOptions).reduce((acum, item) => {
        acum[item] = '';
        return acum;
    }, {});
    const currentField = Object.keys((0, class_transformer_1.plainToClass)(entity, fakeObject)).map((i) => i);
    const relationsFields = Object.keys(relationsOptions);
    const entityName = entity instanceof Function ? entity.name : entity.options.name;
    const newEntity = (0, utils_1.nameIt)(entityName, class {
    });
    return [...currentField, ...relationsFields].reduce((acum, item) => {
        const type = columOptions[item]
            ? Reflect.getMetadata('design:type', entity['prototype'], item)
            : // @ts-ignore
                (0, utils_1.nameIt)(relationsOptions[item]().name, class {
                });
        let isArray = false;
        if (!(item in fakeObject) &&
            Reflect.getMetadata('design:type', entity['prototype'], item) === Array) {
            isArray = true;
        }
        const currentType = type.name === entityName ? acum : type;
        Reflect.defineMetadata('design:type', currentType, acum.prototype, item);
        (0, swagger_1.ApiProperty)({
            required: false,
            isArray: isArray,
            type: () => currentType,
        })(acum.prototype, item);
        return acum;
    }, newEntity);
}
exports.createApiModels = createApiModels;
//# sourceMappingURL=utils.js.map