"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getOne = void 0;
const tslib_1 = require("tslib");
const helper_1 = require("../../../../../helper");
const common_1 = require("@nestjs/common");
function getOne(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const startTime = Date.now();
        const { route: { id: id }, query: { include: include, fields: fields }, } = options;
        const preparedResourceName = (0, helper_1.snakeToCamel)(this.repository.metadata.name);
        if (this.config.requiredSelectField && fields === null) {
            throw new common_1.BadRequestException([
                {
                    source: { parameter: '/fields' },
                    detail: 'Fields params in query is required',
                },
            ]);
        }
        const fieldsSelect = new Set();
        const builder = this.repository.createQueryBuilder(preparedResourceName);
        for (const item of include || []) {
            builder.leftJoin(`${preparedResourceName}.${item}`, item);
            fieldsSelect.add(item);
        }
        fieldsSelect.add(preparedResourceName);
        if (fields) {
            fieldsSelect.clear();
            for (const rel of include || []) {
                const propsName = this.repository.metadata.relations.find((relations) => relations.propertyName === rel).entityMetadata.primaryColumns[0].propertyName;
                fieldsSelect.add(`${rel}.${propsName}`);
            }
            const { target } = fields, other = tslib_1.__rest(fields, ["target"]);
            const targetArray = [...target, this.repository.metadata.primaryColumns[0].propertyName] ||
                [];
            for (const fieldTarget of targetArray) {
                fieldsSelect.add(`${preparedResourceName}.${fieldTarget}`);
            }
            Object.keys(other || {}).forEach((relation) => {
                (other[relation] || []).forEach((i) => {
                    fieldsSelect.add(`${relation}.${i}`);
                });
            });
        }
        const prepareParams = Date.now() - startTime;
        const result = yield builder
            .select([...fieldsSelect])
            .where({ id })
            .getRawOne();
        if (!result) {
            throw new common_1.NotFoundException({
                detail: `Resource '${preparedResourceName}' with id '${id}' does not exist`,
            });
        }
        const callQuery = Date.now() - startTime;
        const itemResult = this.transform.transformRawData([result]);
        const data = this.transform.transformData(itemResult[0], include);
        const included = this.transform.transformInclude(itemResult);
        const transform = Date.now() - startTime;
        const debug = {
            prepareParams,
            callQuery: callQuery - prepareParams,
            transform: transform - callQuery,
        };
        return Object.assign(Object.assign({}, (this.config.debug ? { meta: { debug } } : {})), { data,
            included });
    });
}
exports.getOne = getOne;
//# sourceMappingURL=get-one.js.map