"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRelationship = void 0;
const tslib_1 = require("tslib");
const helper_1 = require("../../../../../helper");
const typeorm_1 = require("typeorm");
const common_1 = require("@nestjs/common");
function getRelationship(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const startTime = Date.now();
        const preparedResourceName = (0, helper_1.snakeToCamel)(this.repository.metadata.name);
        const { relName, id } = options.route;
        const prepareParams = Date.now() - startTime;
        const whereCondition = {
            id: (0, typeorm_1.Equal)(id),
        };
        const findOptionsRelations = {
            [relName]: true,
        };
        const primaryColumns = this.repository.metadata.primaryColumns[0].propertyName;
        const relPrimaryColumns = this.repository.metadata.relations.filter((i) => i.propertyName === relName)[0].inverseEntityMetadata.primaryColumns[0].propertyName;
        const findOptionsSelect = {
            [primaryColumns]: true,
            [relName]: {
                [relPrimaryColumns]: true,
            },
        };
        const callQuery = Date.now() - startTime;
        const result = yield this.repository.findOne({
            select: findOptionsSelect,
            where: whereCondition,
            relations: findOptionsRelations,
        });
        if (!result) {
            throw new common_1.NotFoundException({
                detail: `Resource '${preparedResourceName}' with id '${id}' does not exist`,
            });
        }
        const transform = Date.now() - startTime;
        const debug = {
            prepareParams,
            callQuery: callQuery - prepareParams,
            transform: transform - callQuery,
        };
        return {
            meta: Object.assign({}, (this.config.debug ? { debug } : {})),
            data: this.transform.transformData(result, [relName])['relationships'][relName]['data'],
        };
    });
}
exports.getRelationship = getRelationship;
//# sourceMappingURL=get-relationship.js.map